--[[
The Icarus Project, Copyright (c) 2005 John Forsythe <chaosmachine@gmail.com> (aka chaosmachine)
Feel free to learn and borrow from this code for non-commercial purposes.
If you use my code in your project, I'd appreciate a credit in your readme file.
]]

function getTile(bType, style)
	local yy=0
	
	if bType == "top" then yy=0
	elseif bType == "bottom" then yy=1
	elseif bType == "left" then yy=2
	elseif bType == "right" then yy=3
	elseif bType == "topleft" then yy=4
	elseif bType == "topright" then yy=5
	elseif bType == "bottomleft" then yy=6
	elseif bType == "bottomright" then yy=7
	elseif bType == "topbottom" then yy=8
	elseif bType == "leftright" then yy=9
	elseif bType == "topbottomleft" then yy=10
	elseif bType == "topbottomright" then yy=11
	elseif bType == "topleftright" then yy=12
	elseif bType == "bottomleftright" then yy=13	
	elseif bType == "topbottomleftright" then yy=14
	elseif bType == "cursor" then yy=15
	elseif bType == "goal" then yy=15
	end
	
	local xx=style or 1
	
	if style == "grabby" then xx=4
	elseif style == "border" then xx=5
	elseif bType == "cursor" then xx=0
	elseif bType == "goal" then xx=1
	end
	return {x=xx,y=yy}
end

function initPieceBlockTypes()
	for i,aPiece in ipairs(piece) do
		 getPieceBlockTypes(aPiece)
		 aPiece.lastX = -1
		 aPiece.lastY = -1
	end
end

function getPieceBlockTypes(aPiece)
	-- check the 4 connected blocks to detemine it's tile type
	-- by putting the blocks in a 7x7 grid (no worrying about out-of-bounds error checking)
	
	-- make 7x7 table filled with zeros
	testGrid = {}
	for i=1,7 do
		testGrid[i]={}
		for ii=1,7 do
			testGrid[i][ii] = 0
		end
	end
	
	-- copy the blocks into test graph, offset by +1, for centering purposes
	for i,aBlock in ipairs(aPiece) do
		testGrid[aBlock.x+1][aBlock.y+1] = 1
	end
	
	-- check the surrounding 4 blocks
	for i,aBlock in ipairs(aPiece) do
		bType = ""
		if testGrid[1+aBlock.x][1+aBlock.y+1] == 1 then bType = bType .. "top" end
		if testGrid[1+aBlock.x][1+aBlock.y-1] == 1 then bType = bType .. "bottom" end
		if testGrid[1+aBlock.x+1][1+aBlock.y] == 1 then bType = bType .. "left" end
		if testGrid[1+aBlock.x-1][1+aBlock.y] == 1 then bType = bType .. "right" end
		aBlock.bType = bType
		aBlock.tile = getTile(aBlock.bType, aPiece.style)
	end
end

function initBorderBlockTypes()
	-- check the 4 connected blocks to detemine it's tile type
	-- by putting the blocks in a 32x19 grid (no worrying about out-of-bounds error checking)
	
	-- make 32x19 table filled with zeros
	testGrid = {}
	for i=0,31 do
		testGrid[i]={}
		for ii=0,18 do
			testGrid[i][ii] = 0
		end
	end
	
	-- copy the blocks into test graph, offset by +1, for centering purposes
	for i,aBorder in ipairs(border) do
		testGrid[aBorder.x+1][aBorder.y+1] = 1
	end
	
	-- check the surrounding 4 blocks
	for i,aBorder in ipairs(border) do
		bType = ""
		if testGrid[1+aBorder.x][1+aBorder.y+1] == 1 then bType = bType .. "top" end
		if testGrid[1+aBorder.x][1+aBorder.y-1] == 1 then bType = bType .. "bottom" end
		if testGrid[1+aBorder.x+1][1+aBorder.y] == 1 then bType = bType .. "left" end
		if testGrid[1+aBorder.x-1][1+aBorder.y] == 1 then bType = bType .. "right" end
		aBorder.bType = bType
	end
	--testGrid = nil
end

function generateTiles()
	tiles:clear()
	for i=0,5 do
		-- end blocks
		drawToTiles(i,0,i,4,2)
		drawToTiles(i,1,i,4,0)
		drawToTiles(i,2,i,4,1)
		drawToTiles(i,3,i,4,3)
		-- corner blocks
		drawToTiles(i,4,i,0,0)
		drawToTiles(i,5,i,0,1)
		drawToTiles(i,6,i,0,3)
		drawToTiles(i,7,i,0,2)
		-- straight blocks
		drawToTiles(i,8,i,1,0)
		drawToTiles(i,9,i,1,1)
		-- t blocks
		drawToTiles(i,10,i,2,0)
		drawToTiles(i,11,i,2,2)
		drawToTiles(i,12,i,2,1)
		drawToTiles(i,13,i,2,3)
		-- 4 block
		drawToTiles(i,14,i,3,0)
	end
	
	-- cursor
	drawToTiles(0,15,0,5,0)
	-- goal
	drawToTiles(1,15,1,5,0)
	
-- 	screen:blit(0,0,sourceTiles)
-- 	screen.waitVblankStart()
-- 	screen.flip()
-- 	
-- 	while not Controls.read():select() do
-- 		screen.waitVblankStart()
-- 	end
-- 	
-- 	screen:clear()
-- 	screen:blit(0,0,tiles)
-- 	screen.waitVblankStart()
-- 	screen.flip()
-- 	
-- 	while not Controls.read():square() do
-- 		screen.waitVblankStart()
-- 	end
-- 	
-- 	screen:clear()
	
end